# -*- coding: utf-8 -*-

""" Run local nagios plugins """

import nago.core
from nago.core import nago_access
import nago.settings
import os.path
import pynag.Utils
import subprocess

@nago_access()
def get(search="unsigned"):
    """ List all available plugins"""
    plugins = []
    for i in os.walk('/usr/lib/nagios/plugins'):
        for f in i[2]:
            plugins.append(f)
    return plugins

@nago_access()
def run(plugin_name, *args, **kwargs):
    """ Run a specific plugin """
    plugindir = nago.settings.get_option('plugin_dir')
    plugin = plugindir + "/" + plugin_name
    if not os.path.isfile(plugin):
        raise ValueError("Plugin %s not found" % plugin)

    command = [plugin] + list(args)
    p = subprocess.Popen(command, stdout=subprocess.PIPE,stderr=subprocess.PIPE,)
    stdout, stderr = p.communicate('through stdin to stdout')
    result = {}
    result['stdout'] = stdout
    result['stderr'] = stderr
    result['return_code'] = p.returncode
    return result






import ConfigParser
import platform
import os
import os.path
import errno

_sentinel = object()

cfg_file = "/etc/nago/nago.ini"

defaults = {}
defaults['main'] = {}
defaults['main']['host_name'] = platform.node()
defaults['main']['server'] = 'nago'
defaults['main']['plugin_dir'] = '/usr/lib64/nagios/plugins/'

def get_option(option_name, section_name="main", default=_sentinel, cfg_file=cfg_file):
    """ Returns a specific option specific in a config file

    Arguments:
        option_name  -- Name of the option (example host_name)
        section_name -- Which section of the config (default: name)

    examples:
    >>> get_option("some option", default="default result")
    'default result'
    """
    defaults = get_defaults()

    # As a quality issue, we strictly disallow looking up an option that does not have a default
    # value specified in the code
    #if option_name not in defaults.get(section_name, {}) and default == _sentinel:
    #    raise ValueError("There is no default value for Option %s in section %s" % (option_name, section_name))

    # If default argument was provided, we set variable my_defaults to that
    # otherwise use the global nago defaults
    if default != _sentinel:
        my_defaults = {option_name: default}
    else:
        my_defaults = defaults.get('section_name', {})

    # Lets parse our configuration file and see what we get
    parser = get_parser(cfg_file)
    return parser.get(section_name, option_name, vars=my_defaults)


def get_defaults():
    """ Returns a dictionary of all default options in nago """
    return defaults


def get_parser(cfg_file=cfg_file):
    """ Returns a ConfigParser.ConfigParser() object for our cfg_file """
    if not os.path.exists(cfg_file):
        generate_configfile(cfg_file=cfg_file, defaults=defaults)
    config = ConfigParser.ConfigParser()
    config.read(cfg_file)
    return config


def set_option(section='main', cfg_file=cfg_file, **kwargs):
    """ Change an option in our configuration file """
    parser = get_parser(cfg_file=cfg_file)
    if section not in parser.sections():
        parser.add_section(section)

    for k, v in kwargs.items():
        parser.set(section=section, option=k, value=v)

    with open(cfg_file, 'w') as f:
        parser.write(f)
    return "Done"


def get_section(section_name, cfg_file=cfg_file):
    """ Returns a dictionary of an entire section """
    parser = get_parser(cfg_file=cfg_file)
    options = parser.options(section_name)
    result = {}
    for option in options:
        result[option] = parser.get(section=section_name, option=option)
    return result

def _mkdir_for_config(cfg_file=cfg_file):
    """ Given a path to a filename, make sure the directory exists """
    dirname, filename = os.path.split(cfg_file)
    try:
        os.makedirs(dirname)
    except OSError as exc:
        if exc.errno == errno.EEXIST and os.path.isdir(dirname):
            pass
        else:
            raise


def generate_configfile(cfg_file,defaults=defaults):
    """ Write a new nago.ini config file from the defaults.

    Arguments:
        cfg_file  -- File that is written to like /etc/nago/nago.ini
        defaults  -- Dictionary with default values to use
    """
    # Create a directory if needed and write an empty file
    _mkdir_for_config(cfg_file=cfg_file)
    with open(cfg_file, 'w') as f:
        f.write('')
    for section in defaults.keys():
        set_option(section, cfg_file=cfg_file, **defaults[section])


def get(*args, **kwargs):
    """ Same as get_option() """
    return get_option(*args, **kwargs)


# -*- coding: utf-8 -*-

""" Collect and transmit information about local node """

import nago.core
from nago.core import nago_access
import nago.protocols.httpserver
import nago.settings
import unittest
from collections import defaultdict

node_data = defaultdict(dict)

@nago_access()
def get(security_token=None, key=None):
    """ Get information about this node """
    if security_token is None:
        security_token = nago.core.get_my_info()['host_name']
    data = node_data.get(security_token, {})
    if not key:
        return data
    else:
        return data.get(key)

@nago_access()
def get_all():
    """ Return all information about all nodes """
    return node_data



@nago_access(access_required="node")
def post(node_name, key, **kwargs):
    """ Give the server information about this node

    Arguments:
        node -- node_name or token for the node this data belongs to
        key  -- identifiable key, that you use later to retrieve that piece of data
        kwargs -- the data you need to store
    """
    node = nago.core.get_node(node_name)
    if not node:
        raise ValueError("Node named %s not found" % node_name)
    token = node.token
    node_data[token] = node_data[token] or {}
    node_data[token][key] = kwargs
    return "thanks!"

@nago_access()
def send(node_name):
    """ Send our information to a remote nago instance

    Arguments:
        node -- node_name or token for the node this data belongs to
    """
    my_data = nago.core.get_my_info()
    if not node_name:
        node_name = nago.settings.get('server')
    node = nago.core.get_node(node_name)
    json_params = {}
    json_params['node_name'] = node_name
    json_params['key'] = "node_info"
    for k, v in my_data.items():
        nago.core.log("sending %s to %s" % (k, node['host_name']), level="notice")
        json_params[k] = v
    return node.send_command('info', 'post', node_name=node.token, key="node_info", **my_data)

def on_load():
    my_info = nago.core.get_my_info()
    post(node_name=my_info.get('host_name'), key="my_info", data=my_info)


class TestInfo(unittest.TestCase):
    """ Unit Tests for the info extension """
    def test_get_info(self):
        # Call on_load() to load local info to memory
        on_load()

        # Get our own basic info, double check nago version number
        results = get_info()
        version = results.get('my_info', {}).get('nago_version')
        self.assertEqual(nago.get_version(), version)

        # Try post some changes and see if we can fetch them back
        host_name = results.get('my_info', {}).get('host_name')
        key = 'test'
        data = 'test data'
        tmp = post(host_name, key, data)
        self.assertEqual('thanks!', tmp)

        # get the same data back, make sure it matches
        posted_data = get_info(host_name, key)
        self.assertEqual(data, posted_data)